Pro FITS_REPROJ_HEALPIX, infile, outfile, COORD=crd, PRIMARY=pri, PROJECT=prj, RING=ring, _EXTRA=extra
;+
;   NAME:           
;      FITS_REPROJ_HEALPIX
;   PURPOSE:
;       Convert a HealPix image to a flatmap projection FITS file with WCS info 
;   EXPLANATION:  
;      Writes a FITS file containing a flatmap projection in either a Mollweide
;      or Zenithal Equal Area projection, and write the appropriate World
;      Coordinate System (WCS, see Calabretta & Greisen, 2002, A&A, 395, 1077)
;      information into the FITS header.    This allows coordinates to be 
;      displayed by standard astronomical display software (e.g. DS9, SAOImage).
;
;      Note that the conversion to a flat map projection does not preserve the
;      noise properties of the CMB, and should not be used for a CMB analysis.
;
; CALLING SEQUENCE:
;     FITS_REPROJ_HEALPIX, INFILE, OUTFILE
; INPUT ARGUMENTS:
;       INFILE   - Scalar string giving name of the FITS file containing the
;                  image in HealPix projection.
;                 'map_k_imap_yr1_v1.fits'
;
;       OUTFILE - Scalar string giving the name of the FITS file to contain
;                the flat map projection with astrometry.
;
; INPUT KEYWORDS:
;       COORD   - One of the three words "Galactic" (default), "Celestial", or
;                 "Ecliptic' giving the output projection.   Only the first
;                 letter is needed.
;
;	PRIMARY - If present and nonzero then the first map column will also
;	          be written to the primary HDU.  If there is only one map
;	          column in the input file, then no extensions will be written.
;	          This is only relevant if a Mollweide project is requested.
;
;       PROJECT - Scalar string specifying either a "Mollweide" or a "Zenithal"
;                 Equal Area projection.   Only the first letter is needed.
;	          Defaults to "Mollweide"
;
;	RING    - If present and nonzero then the input map is assumed to be in
;	          RING HEALPIX format.  It is then converted to NESTED format. 
;	          If the input FITS file contains an ORDER card in its header,
;	          that card will override this keyword.
;
;       The following keywords are passed to REPROJ_HEALPIX via _EXTRA:
;       User will be prompted for the following values if they are not supplied
;       as keywords:
;
;       MASK    - Mask outlining the data/nondata ellipse.   Same size as the
;                 the output proj parameter
;
;       SIZE    - Scalar integer giving the size of the output image described by 
;                 the table:
;                       1 -- Small    (512 x 256)
;                       2 -- Medium   (1024 x 512)
;                       3 -- Large    (2048 x 1024)
;                       4 -- X large  (4096 x 2048)
;                       5 -- XX large (8192 x 4096, mollweide, native 
;                            coordinates only)
;	          The smaller dimension controls the size of the square Zenithal
;	          Equal Area projection output maps.
;
;   EXAMPLE:
;       Convert the K band all-sky HealPix map to a medium (1024 x 512) size 
;       Mollweide projection stored in a FITS file moll_k_yr1_v1.fits

;       IDL> infile = 'map_k_imap_yr1_v1.fits'
;       IDL> fits_reproj_healpix, infile, 'moll_k_yr1_v1.fits',size=2,proj='Mol'
;
;   NOTES:
;       A Mollweide projection is written in a single FITS file extension.  A
;       Zenithal Equal Area projection is written as two image in two extensions
;        -- the first centered on the North Pole, and the second on the South 
;       Pole.
;
;	If the input file contains multiple maps (multiple columns in the first
;	FITS extension), each will be written to the output file in a separate
;	FITS image extension.  Each output extension will be given the name of
;	the input column.
;
;	If there is a SKYCOORD keyword in the header, it is used to determine
;	the input map coordinate system.  If it isn't there, GALACTIC coordinates
;	are assumed.
;   PROCEDURES USED:
;       REORDER_STRUCT(), REPROJ_HEALPIX
;   MODIFICATION HISTORY:
;	Written by Wayne Landsman.
;	Multiple map support.  Michael R. Greason, ADNET, 15 June 2009.
;	PRIMARY and RING added.  Rewritten to use REPROJ_HEALPIX.  MRG, ADNET,
;	   19 February 2010.
;-
on_error, 2
;
;			Check arguments.
;
If (n_params() LE 1) Then message, "Syntax - fits_reproj_healpix, infile, outfile"
;
If (n_elements(crd) LE 0) Then crd = 'GALACTIC'					$
                          Else crd = strupcase(crd)
Case (strmid(crd, 0, 1)) Of
   'E' : Begin			; Ecliptic output.
         lcoord = 3
	 ctype  = ['ELON','ELAT']
	 End
   'C' : Begin			; Celestial output.
         lcoord = 2
	 ctype  = ['RA--','DEC-']
	 End
  Else : Begin			; Galactic output.
         lcoord = 1
	 ctype  = ['GLON','GLAT']
	 End
EndCase
;
If (n_elements(prj) LE 0) Then prj = 'MOLLWEIDE'				$
                          Else prj = strupcase(prj)
Case (strmid(prj, 0, 1)) Of
   'Z' : Begin			; Zenithal Equal Area output.
         lprj  = 2
	 ctype = ctype + '-ZEA'
	 End
  Else : Begin			; Mollweide output.
         lprj  = 1
	 ctype = ctype + '-MOL'
	 crval = [0.0, 0.0]
	 End
EndCase
;
crvaln = [ 90.0,  90.0]
crvals = [270.0, -90.0]
crvalm = [  0.0,   0.0]
;
;			Read the input file.
;
img = mrdfits(infile, 0, ph, /Silent)
tab = mrdfits(infile, 1, eh, /Silent)
;
;			Determine the coordinate conversion by checking for the
;			SKYCOORD keyword in the header.  If it doesn't exist,
;			assume that the input file is in Galactic coordinates.
;
icrd = sxpar(ph, 'SKYCOORD', Count=cnt, /Silent)
If (cnt GT 0) Then Begin
  icrd = strupcase(strmid(icrd, 0, 1))
  Case (icrd) Of
     'E' : lcoord = lcoord + 6		; Ecliptic input.
     'C' : lcoord = lcoord + 3		; Celestial input.
    Else : lcoord = lcoord		; Galactic input.
  EndCase
EndIf
;
;			Reorder the map from RING to NESTED.  At the same time,
;			force the map array to contain a single pixel per element.
;
ord  = strupcase(sxpar(ph, 'ORDER', Count=cnt, /Silent))
oflg = 0
If (cnt LE 0) Then Begin
  If (keyword_set(ring)) Then oflg = 1
EndIf Else Begin
  If (ord EQ 'RING') Then oflg = 1
EndElse
tab  = reorder_struct(tab, R2N=oflg)
tags = tag_names(tab)
ntg  = n_elements(tags) - 1L
;
;			Write the primary HDU.
;
SXADDHIST, ['FITS_REPROJ_HEALPIX:' + systime() + $
   ' Converted to Flat Map Projection', $
    'This FITS file contains a reprojection of WMAP HealPix data into',		$
     'a flat map; this changes the noise properties of the CMB data.  ',	$
     'Therefore this map should not be used for a CMB analysis'], ph
;
If ((lprj EQ 1) AND (keyword_set(pri))) Then Begin
  REPROJ_HEALPIX, tab.(0), img, Coord=lcoord, Project=lprj, _EXTRA=extra
  dimen = size(img, /dimen)
  crpix = dimen / 2.0D0 + 0.5D0
  cdelt = 360.0d * sqrt(2.0d) / !dpi * [(-2. / dimen[0]), (1. / dimen[1])]
  make_astr, astr, Ctype=ctyp, Delt=cdelt, CRpix=crpix, CRval=crvalm, LongPole=0.0
  putast, ph, astr, CD_Type=1
EndIf
mwrfits, img, outfile, ph, /Create, /Silent
If ((n_elements(img) GT 1) AND (ntg LE 0)) Then Return
;
;			Write the extensions.
;
For i = 0, ntg Do Begin
  REPROJ_HEALPIX, tab.(i), img, Coord=lcoord, Project=lprj, _EXTRA=extra
  dimen = size(img, /dimen)
  If (lprj EQ 2) Then Begin
    imsize = dimen[1]
    imsiz2 = imsize / 2.0
    img_n = img[0:imsize-1,*]
    img_s = img[imsize:*,*]
    mkhdr, hn, img_n, /Image
    mkhdr, hs, img_s, /Image
    sxaddpar, hn, 'EXTNAME', strtrim(tags[i], 2) + '_NORTH'
    sxaddpar, hs, 'EXTNAME', strtrim(tags[i], 2) + '_SOUTH'
    crpix = replicate(imsiz2 + 0.5, 2)
    cdelt = 180 * sqrt(2) / (imsize2 * !pi) * [-1,1]
    make_astr, astr, Ctype=ctype, Delt=cdelt, CRpix=crpix, CRval=crvaln, LongPole=0.0
    putast, hn, astr, CD_Type=1
    cdelt = -cdelt
    make_astr, astr, Ctype=ctype, Delt=cdelt, CRpix=crpix, CRval=crvals, LongPole=180.0
    putast, hs, astr, CD_Type=1
    mwrfits, img_n, outfile, hn, /Silent
    mwrfits, img_s, outfile, hs, /Silent
  EndIf Else Begin
    mkhdr, h, img, /Image
    sxaddpar, h, 'EXTNAME', tags[i]
    crpix = dimen / 2.0D0 + 0.5D0
    cdelt = 360.0d * sqrt(2.0d) / !dpi * [(-2. / dimen[0]), (1. / dimen[1])]
    make_astr, astr, Ctype=ctyp, Delt=cdelt, CRpix=crpix, CRval=crvalm, LongPole=0.0
    putast, ph, astr, CD_Type=1
    mwrfits, img, outfile, h, /Silent
  EndElse
EndFor
;
Return
End 
