Function reorder_struct, itab, IN=in, OUT=out, N2R=n2r, R2N=r2n
;+
; NAME:
;	reorder_struct
; PURPOSE:
;	This function restructs a HEALPIX skymap input array such that each element
;	contains a single pixel.  It also will convert the indexing between RING
;	and NESTED.
; CALLING SEQUENCE:
;	otab = reorder_struct(itab)
; INPUTS:
;	itab - The input skymap array.
; RETURNED:
;	otab - The restructured skymap array.
; KEYWORDS:
;	in  - The input ordering: RING or NESTED.
;	out - The output ordering: RING or NESTED.
;	n2r - A shortcut, setting in to NESTED and out to RING.
;	r2n - A shortcut, setting in to RING and out to NESTED.
; COMMENTS:
;	The output array will be constructed such that each element will contain
;	one pixel.  Then the reordering will take place according to the following
;	priorities:
; MODIFICATION HISTORY:
;	Written by Michael R. Greason, ADNET, 19 February 2010.
;-
on_error, 2
;
;			Check arguments.
;
If (n_params() LT 1) Then message, 'Syntax:  otab = reorder_struct(itab)'
;
If (n_elements(in)  LE 0) Then in  = ''
If (n_elements(out) LE 0) Then out = ''
;
;			Construct the output array such that each element contains
;			a single pixel.
;
tags = tag_names(itab)
n = n_elements(tags) - 1L
Case n Of		; Overkill, I know...
  0 : ot = create_struct(tags, 0.0)
  1 : ot = create_struct(tags, 0.0, 0.0)
  2 : ot = create_struct(tags, 0.0, 0.0, 0.0)
  3 : ot = create_struct(tags, 0.0, 0.0, 0.0, 0.0)
  4 : ot = create_struct(tags, 0.0, 0.0, 0.0, 0.0, 0.0)
  5 : ot = create_struct(tags, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0)
  6 : ot = create_struct(tags, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0)
  7 : ot = create_struct(tags, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0)
  Else : message, 'Invalid input table format.'
EndCase
;
m = n_elements(itab.(0))
otab = replicate(ot, m)
;
;			Copy the input array into the output array, reordering as needed.
;
For i = 0L, n Do Begin
;
  t = itab.(i)
;
  If (keyword_set(r2n)) Then Begin
    otab.(i) = reorder(t[*], /R2N)
  EndIf Else If (keyword_set(n2r)) Then Begin
    otab.(i) = reorder(t[*], /N2R)
  EndIf Else If (((strupcase(in)  EQ 'NESTED') OR (strupcase(in)  EQ 'RING')) $
             AND ((strupcase(out) EQ 'NESTED') OR (strupcase(out) EQ 'RING'))) Then Begin
    otab.(i) = reorder(t[*], IN=in, OUT=out)
  EndIf Else Begin
    otab.(i) = t[*]
  EndElse
EndFor
;
;			Done.
;
Return, otab
End
